# -*- coding: utf-8 -*-
 
"""
file   chow.py
 
author Dr. Ernesto P. Adorio
       UPDEPP at Pampanga(UP Clarkfield)
       email: ernesto.adorio@gmail.com
 
       educational use.
 
revision
       October 8, 2009  First version.
"""
 
#from scipy import stats
import lm
import rpy
import numpy

stats_f_ppf = rpy.r.qf

def chow(X, Y, breakpoint, alpha = 0.05):
    """
    Performs a chow test.
    Split input matrix and output vector into two
    using specified breakpoint.
      X - independent variables matrix
      Y - dependent variable vector
      breakpoint -index to split.
      alpha is significance level
    """
    k = len(X[0])
    n = len(X)
 
    # Split into two datasets.
    X1 = X[:breakpoint][:]
    Y1 = Y[:breakpoint][:]
 
    X2 = X[breakpoint:][:]
    Y2 = Y[breakpoint:][:]
 
    # Perform separate three least squares.
    allfit   = lm.ols(X,Y)
    lowerfit = lm.ols(X1, Y1)
    upperfit = lm.ols(X2, Y2)
 
    RSS  = allfit.RSS
    RSS1 = lowerfit.RSS
    RSS2 = upperfit.RSS
 
    df1 = k
    df2 = n - 2 *k
 
    num = (RSS - (RSS1 + RSS2)) /float(df1)
    den = (RSS1 + RSS2) / (df2)
 
 
    Ftest = num/den    
    Fcrit = stats_f_ppf(1 -0.05, df1, df2)
    Pvalue = 1-rpy.r.pf(Ftest,df1,df2)
    return (Pvalue, Ftest, Fcrit, df1, df2, RSS, RSS1, RSS2)
 
def normalized_chow_test( x1, y1, x2, y2, alpha = 0.05 ):
   X = []
   Y = []
   mean_x1 = numpy.mean(x1)
   mean_y1 = numpy.mean(y1)
   for (i,w) in enumerate(x1):
      X.append([1.0, w - mean_x1])
      Y.append(y1[i] - mean_y1)
   breakpoint = i + 1

   mean_x2 = numpy.mean(x2)
   mean_y2 = numpy.mean(y2)
   for (i,w) in enumerate(x2):
      X.append([1.0, w - mean_x2])
      Y.append(y2[i] - mean_y2)
    
   return chow( X, Y, breakpoint, alpha )

def chow_test( x1, y1, x2, y2, alpha = 0.05 ):
   X = []
   Y = []
   for (i,w) in enumerate(x1):
      X.append([1.0, w])
      Y.append(y1[i])
   breakpoint = i + 1

   for (i,w) in enumerate(x2):
      X.append([1.0, w])
      Y.append(y2[i])
    
   return chow( X, Y, breakpoint, alpha )
    
if __name__ == "__main__":
   # linear equation to fit is pcer = const +  gnpr
   data = """year    pcer    gnpr
1975    311.848 452.735
1976    327.436 489.471
1977    344.706 517.88
1978    362.676 545.516
1979    380.263 579.576
1980    397.404 605.687
1981    407.958 626.321
1982    422.067 641.51
1983    424.634 652.097
1984    425.767 592.694
1985    420.832 551.428
1986    434.815 571.492
1987    452.386 600.907
1988    480.562 644.229
1989    504.619 684.231
1990    531.772 716.929
1991    543.788 720.218
1992    561.509 731.396
1993    578.589 746.921
1994    600.106 786.136
1995    622.985 825.164
1996    651.79  882.399"""
 
   data2 = """year    pcer    gnpr
1975	-63.59555556	-115.1308889
1976	-48.00755556	-78.39488889
1977	-30.73755556	-49.98588889
1978	-12.76755556	-22.34988889
1979	4.819444444	11.71011111
1980	21.96044444	37.82111111
1981	32.51444444	58.45511111
1982	46.62344444	73.64411111
1983	49.19044444	84.23111111
1984	-98.04223077	-103.7786154
1985	-102.9772308	-145.0446154
1986	-88.99423077	-124.9806154
1987	-71.42323077	-95.56561538
1988	-43.24723077	-52.24361538
1989	-19.19023077	-12.24161538
1990	7.962769231	20.45638462
1991	19.97876923	23.74538462
1992	37.69976923	34.92338462
1993	54.77976923	50.44838462
1994	76.29676923	89.66338462
1995	99.17576923	128.6913846
1996	127.9807692	185.9263846"""

 
   W = data2.split("\n")[1:] #ignore first line
   X = []
   Y = []
   for (i,w) in enumerate(W):
      w = w.split()
      X.append([1.0, float(w[2])])
      Y.append(float(w[1]))
 
   # for (x,y) in zip(X,Y):
   #   print x[0], x[1],y
 
   breakpoint = 9
   Pvalue, Ftest, Fcrit, df1, df2, RSS, RSS1, RSS2 = \
      chow(X, Y, breakpoint, alpha = 0.05)
   print "Chow Summary statistics:"
   print "Breakpoint index,(zero based)= ", breakpoint
   print "Number of indep. vars(incl. const), k =", len(X[0])
   print "RSS1=", RSS1, "n1=", breakpoint
   print "RSS2=", RSS2,"n2=", len(X) - breakpoint
   print "RSS =", RSS, "n=", len(X)
   print "df1,df2=", df1, df2  
   print "F values:(test, crit)=", Ftest, Fcrit
   print "P value=", Pvalue

   y1 = [311.848,327.436,344.706,362.676,380.263,397.404,407.958,422.067,424.634,]
   x1 = [452.735,489.471,517.88,545.516,579.576,605.687,626.321,641.51,652.097,]
   y2 = [425.767,420.832,434.815,452.386,480.562,504.619,531.772,543.788,561.509,578.589,600.106,622.985,651.79,]
   x2 = [592.694,551.428,571.492,600.907,644.229,684.231,716.929,720.218,731.396,746.921,786.136,825.164,882.399,]
   print chow_test( x1, y1, x2, y2 )
   print normalized_chow_test( x1, y1, x2, y2 )
